# Copyright (c) 2023 NEC Corporation. All Rights Reserved.

from abc import abstractmethod
import pandas
from fireducks import ir, irutils
from fireducks.pandas import utils, DataFrame, Series


# https://pandas.pydata.org/docs/reference/window.html
_known_funcs = {
    "count": "count",
    "sum": "sum",
    "mean": "mean",
    "median": "median",
    "var": "variance",
    "std": "stddev",
    "min": "min",
    "max": "max",
    "corr": None,
    "skew": None,
    "kurt": None,
    "quantile": None,
    "sem": None,
    "rank": None,
}


def to_supported_function(func):
    return _known_funcs.get(func)


class Rolling:
    def __init__(self, obj, args, kwargs):
        self._obj = obj
        self._rolling_args = args
        self._rolling_kwargs = kwargs

    def _unwrap(self, reason=None):
        return self._obj._fallback_call_packed(
            "rolling", self._rolling_args, self._rolling_kwargs, reason=reason
        )

    def _fallback_call(self, method, args, kwargs, *, reason=None):
        return utils.fallback_call_packed(
            self._unwrap, method, args, kwargs, reason=reason
        )

    def aggregate(self, *args, **kwargs):
        pandas_cls = utils._pandas_class(self._obj.__class__)
        ns = utils.decode_args(
            args, kwargs, pandas.core.window.rolling.Rolling.aggregate
        )

        rolling_ns = utils.decode_args(
            self._rolling_args, self._rolling_kwargs, pandas_cls.rolling
        )

        if not isinstance(rolling_ns.window, int):
            reason = "window is not int"
        else:
            reason = rolling_ns.is_not_default(
                [
                    "center",
                    "win_type",
                    "on",
                    "axis",
                    "closed",
                    "step",
                    "method",
                ]
            )

        if isinstance(ns.func, str):
            funcs = to_supported_function(ns.func)
            if funcs is None:
                reason = f"unsupported function: '{ns.func}'"
        elif isinstance(ns.func, (list, tuple)):
            if len(ns.func) == 0:
                raise ValueError("no results")
            reason = "list of function is not supported"

        if rolling_ns.min_periods is None:
            if isinstance(rolling_ns.window, int):
                if utils._pd_version_under2:
                    rolling_ns.min_periods = (
                        0 if ns.func == "count" else rolling_ns.window
                    )
                else:
                    rolling_ns.min_periods = rolling_ns.window
            else:
                reason = (
                    "unsupported min_periods=None with "
                    f"window of type: '{type(rolling_ns.window)}'"
                )
        elif isinstance(rolling_ns.min_periods, int):
            if rolling_ns.min_periods > rolling_ns.window:
                raise ValueError(
                    f"min_periods {rolling_ns.min_periods} must be"
                    f"<= window {rolling_ns.window}"
                )
        else:
            reason = f"unsupported min_periods: '{rolling_ns.min_periods}'"

        if reason:
            return self._fallback_call(
                "aggregate", args, kwargs, reason=reason
            )

        window = rolling_ns.window
        funcs = irutils.make_vector_or_scalar_of_str(funcs)
        return self._obj.__class__._create(
            ir.rolling_aggregate(
                self._obj._value, window, rolling_ns.min_periods, funcs
            )
        )

    agg = aggregate

    def count(self):
        return self.aggregate("count")

    def sum(self):
        return self.aggregate("sum")

    def mean(self):
        return self.aggregate("mean")

    def median(self):
        return self.aggregate("median")

    def min(self):
        return self.aggregate("min")

    def max(self):
        return self.aggregate("max")


# as of now there is no specific method to implement separately.
# keeping these two classes as placeholder, like in pandas
class DataFrameRolling(Rolling):
    pass


class SeriesRolling(Rolling):
    pass
